package speiger.src.coreengine.rendering.gui.font;

import java.util.List;

import speiger.src.collections.ints.maps.impl.hash.Int2ObjectOpenHashMap;
import speiger.src.collections.ints.maps.interfaces.Int2ObjectMap;
import speiger.src.coreengine.assets.AssetLocation;
import speiger.src.coreengine.rendering.gui.font.glyth.Glyth;
import speiger.src.coreengine.rendering.gui.font.glyth.GlythData;
import speiger.src.coreengine.rendering.gui.font.glyth.GlythData.GlythBaker;
import speiger.src.coreengine.rendering.gui.font.providers.IFontProvider;

public class FontGroup {
	AssetLocation locations;
	List<IFontProvider> providers;
	GlythBaker baker;
	StyledFont[] cache = new StyledFont[] {new StyledFont(0), new StyledFont(1), new StyledFont(2), new StyledFont(3)};
	
	public FontGroup(AssetLocation locations, List<IFontProvider> providers, GlythBaker baker) {
		this.locations = locations;
		this.providers = providers;
		this.baker = baker;
	}
	
	public GlythData data(int codepoint, int style) {
		return cache[style & 0x3].data(codepoint);
	}
	
	public Glyth glyth(int codepoint, int style) {
		return cache[style & 0x3].glyth(codepoint);
	}
	
	public void close() {
		providers.forEach(IFontProvider::close);
		providers.clear();
	}
	
	private class StyledFont {
		final int style;
		Int2ObjectMap<Glyth> bakedGlyths = new Int2ObjectOpenHashMap<>();
		Int2ObjectMap<GlythData> dataGlyths = new Int2ObjectOpenHashMap<>();
		
		public StyledFont(int style) {
			this.style = style;
		}
		
		public GlythData data(int codepoint) { 
			return dataGlyths.computeIfAbsent(codepoint, this::compute);
		}
		
		public Glyth glyth(int codepoint) {
			return bakedGlyths.computeIfAbsent(codepoint, this::bake);
		}
		
		private GlythData compute(int codepoint) {
			for(int i = 0,m=providers.size();i<m;i++) {
				GlythData data = providers.get(i).glythData(codepoint, style);
				if(data != null) return data;
			}
			return null;
		}
		
		private Glyth bake(int codepoint) {
			for(int i = 0,m=providers.size();i<m;i++) {
				GlythData data = providers.get(i).glythData(codepoint, style);
				if(data != null) return data.bake(baker);
			}
			return null;
		}
	}
}